@@ -1,6 +1,5 @@
 """
-Defines a collection of fields to represent multiple fields defined on a common
-grid.
+Defines a collection of fields to represent multiple fields defined on a common grid.
 
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
@@ -9,7 +8,17 @@ from __future__ import annotations
 
 import json
 import logging
-from typing import Any, Dict, Iterator, List, Optional, Sequence, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Union,
+    overload,
+)
 
 import numpy as np
 
@@ -140,19 +149,38 @@ class FieldCollection(FieldBase):
         """return iterator over the actual fields"""
         return iter(self.fields)
 
+    @overload
     def __getitem__(self, index: Union[int, str]) -> DataFieldBase:
-        """return a specific field"""
+        ...
+
+    @overload
+    def __getitem__(self, index: slice) -> FieldCollection:
+        ...
+
+    def __getitem__(
+        self, index: Union[int, str, slice]
+    ) -> Union[DataFieldBase, FieldCollection]:
+        """returns one or many fields from the collection
+
+        If `index` is an integer or string, the field at this position or with this
+        label is returned, respectively. If `index` is a :class:`slice`, a collection is
+        returned. In this case the field data is copied.
+        """
         if isinstance(index, int):
-            # simple numerical index
+            # simple numerical index -> return single field
             return self.fields[index]
 
         elif isinstance(index, str):
-            # index specifying the label of the field
+            # index specifying the label of the field -> return a single field
             for field in self.fields:
                 if field.label == index:
                     return field
             raise KeyError(f"No field with name `{index}`")
 
+        elif isinstance(index, slice):
+            # range of indices -> collection is returned
+            return FieldCollection(self.fields[index], copy_fields=True)
+
         else:
             raise TypeError(f"Unsupported index `{index}`")
 
@@ -372,7 +400,7 @@ class FieldCollection(FieldBase):
         ]
 
         # create vector field from the data
-        return cls(fields=fields, label=label)  # lgtm [py/call-to-non-callable]
+        return cls(fields=fields, label=label)
 
     @classmethod
     def scalar_random_uniform(
@@ -483,6 +511,19 @@ class FieldCollection(FieldBase):
         # create the collection from the copied fields
         return self.__class__(fields, copy_fields=False, label=label, dtype=dtype)
 
+    def _unary_operation(self: FieldCollection, op: Callable) -> FieldCollection:
+        """perform an unary operation on this field collection
+
+        Args:
+            op (callable):
+                A function calculating the result
+
+        Returns:
+            FieldBase: An field that contains the result of the operation.
+        """
+        fields = [fields._unary_operation(op) for fields in self.fields]
+        return self.__class__(fields, label=self.label)
+
     def interpolate_to_grid(
         self,
         grid: GridBase,
@@ -624,7 +665,6 @@ class FieldCollection(FieldBase):
     def plot(
         self,
         kind: Union[str, Sequence[str]] = "auto",
-        resize_fig=None,
         figsize="auto",
         arrangement="horizontal",
         fig=None,
@@ -639,8 +679,6 @@ class FieldCollection(FieldBase):
                 `line`, `vector`, or `interactive`. Alternatively, `auto` determines the
                 best visualization based on each field itself. Instead of a single value
                 for all fields, a list with individual values can be given.
-            resize_fig (bool):
-                Whether to resize the figure to adjust to the number of panels
             figsize (str or tuple of numbers):
                 Determines the figure size. The figure size is unchanged if the string
                 `default` is passed. Conversely, the size is adjusted automatically when
@@ -663,22 +701,6 @@ class FieldCollection(FieldBase):
             List of :class:`PlotReference`: Instances that contain information
             to update all the plots with new data later.
         """
-        if resize_fig is not None:
-            # Deprecated this argument on 2021-02-01
-            import warnings
-
-            warnings.warn(
-                "`resize_fig` is a deprecated argument. Use `figsize` directly",
-                DeprecationWarning,
-            )
-
-            if resize_fig is True:
-                figsize = "auto"
-            elif resize_fig is False:
-                figsize = "default"
-            else:
-                raise ValueError
-
         # set the size of the figure
         if figsize == "default":
             pass  # just leave the figure size at its default value
