@@ -62,9 +62,7 @@ class Boundaries(list):
         return f"[{items}]"
 
     @classmethod
-    def from_data(
-        cls, grid: GridBase, boundaries, rank: int = 0, normal: bool = False
-    ) -> Boundaries:
+    def from_data(cls, grid: GridBase, boundaries, rank: int = 0) -> Boundaries:
         """
         Creates all boundaries from given data
 
@@ -82,9 +80,6 @@ class Boundaries(list):
                 * tuple pair specifying the low and high boundary individually
             rank (int):
                 The tensorial rank of the field for this boundary condition
-            normal (bool):
-                Flag indicating whether the condition is only applied in the normal
-                direction.
         """
         # check whether this is already the correct class
         if isinstance(boundaries, Boundaries):
@@ -132,7 +127,7 @@ class Boundaries(list):
         if isinstance(boundaries, (str, dict)):
             # one specification for all axes
             bcs = [
-                get_boundary_axis(grid, i, boundaries, rank=rank, normal=normal)
+                get_boundary_axis(grid, i, boundaries, rank=rank)
                 for i in range(grid.num_axes)
             ]
 
@@ -141,12 +136,12 @@ class Boundaries(list):
             if len(boundaries) == grid.num_axes:
                 # assume that data is given for each boundary
                 bcs = [
-                    get_boundary_axis(grid, i, boundary, rank=rank, normal=normal)
+                    get_boundary_axis(grid, i, boundary, rank=rank)
                     for i, boundary in enumerate(boundaries)
                 ]
             elif grid.num_axes == 1 and len(boundaries) == 2:
                 # special case where the two sides can be specified directly
-                bcs = [get_boundary_axis(grid, 0, boundaries, rank=rank, normal=normal)]
+                bcs = [get_boundary_axis(grid, 0, boundaries, rank=rank)]
 
         if bcs is None:
             # none of the logic worked
@@ -161,10 +156,6 @@ class Boundaries(list):
             return NotImplemented
         return super().__eq__(other) and self.grid == other.grid
 
-    def _cache_hash(self) -> int:
-        """returns a value to determine when a cache needs to be updated"""
-        return hash(tuple(bc_ax._cache_hash() for bc_ax in self))
-
     def check_value_rank(self, rank: int) -> None:
         """check whether the values at the boundaries have the correct rank
 
@@ -265,16 +256,6 @@ class Boundaries(list):
 
         return "\n".join(result)
 
-    def extract_component(self, *indices) -> Boundaries:
-        """extracts the boundary conditions of the given component of the tensor.
-
-        Args:
-            *indices:
-                One or two indices for vector or tensor fields, respectively
-        """
-        boundaries = [boundary.extract_component(*indices) for boundary in self]
-        return self.__class__(boundaries)
-
     def set_ghost_cells(self, data_full: np.ndarray, *, args=None) -> None:
         """set the ghost cells for all boundaries
 
