@@ -11,17 +11,23 @@ non-periodic axes have more option, which are represented by
 
 from __future__ import annotations
 
-from typing import Dict, Tuple, Union
+from typing import TYPE_CHECKING, Dict, Tuple, Union
 
 import numpy as np
 from numba.extending import register_jitable
 
 from ...tools.typing import GhostCellSetter
 from ..base import GridBase, PeriodicityError
-from .local import BCBase, BCDataError, BoundaryData, _PeriodicBC
+from .local import _MPIBC, BCBase, BCDataError, BoundaryData, _PeriodicBC
+
+if TYPE_CHECKING:
+    from .._mesh import GridMesh  # @UnusedImport
 
 BoundaryPairData = Union[
-    Dict[str, BoundaryData], BoundaryData, Tuple[BoundaryData, BoundaryData]
+    Dict[str, BoundaryData],
+    BoundaryData,
+    Tuple[BoundaryData, BoundaryData],
+    "BoundaryAxisBase",
 ]
 
 
@@ -58,6 +64,27 @@ class BoundaryAxisBase:
         self.low = low
         self.high = high
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self.low!r}, {self.high!r})"
+
+    def __str__(self):
+        if self.low == self.high:
+            return str(self.low)
+        else:
+            return f"({self.low}, {self.high})"
+
+    @classmethod
+    def get_help(cls) -> str:
+        """Return information on how boundary conditions can be set"""
+        return (
+            "Boundary conditions for each side can be set using a tuple: "
+            f"(lower_bc, upper_bc). {BCBase.get_help()}"
+        )
+
+    def copy(self) -> BoundaryAxisBase:
+        """return a copy of itself, but with a reference to the same grid"""
+        return self.__class__(self.low.copy(), self.high.copy())
+
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
@@ -101,7 +128,6 @@ class BoundaryAxisBase:
             upper=upper,
             data=data,
             rank=self[upper].rank,
-            normal=self[upper].normal,
         )
 
         # set the data
@@ -170,20 +196,32 @@ class BoundaryAxisBase:
                 Additional arguments that might be supported by special boundary
                 conditions.
         """
-        self.low.set_ghost_cells(data_full, args=args)
+        # send boundary information to other nodes if using MPI
+        if isinstance(self.low, _MPIBC):
+            self.low.send_ghost_cells(data_full, args=args)
+        if isinstance(self.high, _MPIBC):
+            self.high.send_ghost_cells(data_full, args=args)
+        # set the actual ghost cells
         self.high.set_ghost_cells(data_full, args=args)
+        self.low.set_ghost_cells(data_full, args=args)
 
     def make_ghost_cell_setter(self) -> GhostCellSetter:
         """return function that sets the ghost cells for this axis on a full array"""
-
+        # get the functions that handle the data
+        ghost_cell_sender_low = self.low.make_ghost_cell_sender()
+        ghost_cell_sender_high = self.high.make_ghost_cell_sender()
         ghost_cell_setter_low = self.low.make_ghost_cell_setter()
         ghost_cell_setter_high = self.high.make_ghost_cell_setter()
 
         @register_jitable
         def ghost_cell_setter(data_full: np.ndarray, args=None) -> None:
             """helper function setting the conditions on all axes"""
-            ghost_cell_setter_low(data_full, args=args)
+            # send boundary information to other nodes if using MPI
+            ghost_cell_sender_low(data_full, args=args)
+            ghost_cell_sender_high(data_full, args=args)
+            # set the actual ghost cells
             ghost_cell_setter_high(data_full, args=args)
+            ghost_cell_setter_low(data_full, args=args)
 
         return ghost_cell_setter  # type: ignore
 
@@ -191,35 +229,8 @@ class BoundaryAxisBase:
 class BoundaryPair(BoundaryAxisBase):
     """represents the two boundaries of an axis along a single dimension"""
 
-    def __repr__(self):
-        return f"{self.__class__.__name__}({self.low!r}, {self.high!r})"
-
-    def __str__(self):
-        if self.low == self.high:
-            return str(self.low)
-        else:
-            return f"({self.low}, {self.high})"
-
-    def _cache_hash(self) -> int:
-        """returns a value to determine when a cache needs to be updated"""
-        return hash((self.low._cache_hash(), self.high._cache_hash()))
-
-    def copy(self) -> BoundaryPair:
-        """return a copy of itself, but with a reference to the same grid"""
-        return self.__class__(self.low.copy(), self.high.copy())
-
     @classmethod
-    def get_help(cls) -> str:
-        """Return information on how boundary conditions can be set"""
-        return (
-            "Boundary conditions for each side can be set using a tuple: "
-            f"(lower_bc, upper_bc). {BCBase.get_help()}"
-        )
-
-    @classmethod
-    def from_data(
-        cls, grid: GridBase, axis: int, data, rank: int = 0, normal: bool = False
-    ) -> BoundaryPair:
+    def from_data(cls, grid: GridBase, axis: int, data, rank: int = 0) -> BoundaryPair:
         """create boundary pair from some data
 
         Args:
@@ -231,9 +242,6 @@ class BoundaryPair(BoundaryAxisBase):
                 Data that describes the boundary pair
             rank (int):
                 The tensorial rank of the field for this boundary condition
-            normal (bool):
-                Flag indicating whether the condition is only applied in the normal
-                direction.
 
         Returns:
             :class:`~pde.grids.boundaries.axis.BoundaryPair`:
@@ -248,32 +256,20 @@ class BoundaryPair(BoundaryAxisBase):
                 # separate conditions for low and high
                 data_copy = data.copy()
                 d_low = data_copy.pop("low")
-                low = BCBase.from_data(
-                    grid, axis, upper=False, data=d_low, rank=rank, normal=normal
-                )
+                low = BCBase.from_data(grid, axis, upper=False, data=d_low, rank=rank)
                 d_high = data_copy.pop("high")
-                high = BCBase.from_data(
-                    grid, axis, upper=True, data=d_high, rank=rank, normal=normal
-                )
+                high = BCBase.from_data(grid, axis, upper=True, data=d_high, rank=rank)
                 if data_copy:
                     raise BCDataError(f"Data items {data_copy.keys()} were not used.")
             else:
                 # one condition for both sides
-                low = BCBase.from_data(
-                    grid, axis, upper=False, data=data, rank=rank, normal=normal
-                )
-                high = BCBase.from_data(
-                    grid, axis, upper=True, data=data, rank=rank, normal=normal
-                )
+                low = BCBase.from_data(grid, axis, upper=False, data=data, rank=rank)
+                high = BCBase.from_data(grid, axis, upper=True, data=data, rank=rank)
 
         elif isinstance(data, (str, BCBase)):
             # a type for both boundaries
-            low = BCBase.from_data(
-                grid, axis, upper=False, data=data, rank=rank, normal=normal
-            )
-            high = BCBase.from_data(
-                grid, axis, upper=True, data=data, rank=rank, normal=normal
-            )
+            low = BCBase.from_data(grid, axis, upper=False, data=data, rank=rank)
+            high = BCBase.from_data(grid, axis, upper=True, data=data, rank=rank)
 
         else:
             # the only remaining valid format is a list of conditions for the
@@ -290,10 +286,10 @@ class BoundaryPair(BoundaryAxisBase):
                 if data_len == 2:
                     # assume that data is given for each boundary
                     low = BCBase.from_data(
-                        grid, axis, upper=False, data=data[0], rank=rank, normal=normal
+                        grid, axis, upper=False, data=data[0], rank=rank
                     )
                     high = BCBase.from_data(
-                        grid, axis, upper=True, data=data[1], rank=rank, normal=normal
+                        grid, axis, upper=True, data=data[1], rank=rank
                     )
                 else:
                     # if the length is strange, the format must be wrong
@@ -304,17 +300,6 @@ class BoundaryPair(BoundaryAxisBase):
 
         return cls(low, high)
 
-    def extract_component(self, *indices) -> BoundaryPair:
-        """extracts the boundary pair of the given index.
-
-        Args:
-            *indices:
-                One or two indices for vector or tensor fields, respectively
-        """
-        bc_sub_low = self.low.extract_component(*indices)
-        bc_sub_high = self.high.extract_component(*indices)
-        return self.__class__(bc_sub_low, bc_sub_high)
-
     def check_value_rank(self, rank: int) -> None:
         """check whether the values at the boundaries have the correct rank
 
@@ -364,23 +349,10 @@ class BoundaryPeriodic(BoundaryPair):
         else:
             return '"periodic"'
 
-    def _cache_hash(self) -> int:
-        """returns a value to determine when a cache needs to be updated"""
-        return hash((self.grid._cache_hash(), self.axis, self.flip_sign))
-
     def copy(self) -> BoundaryPeriodic:
         """return a copy of itself, but with a reference to the same grid"""
         return self.__class__(grid=self.grid, axis=self.axis, flip_sign=self.flip_sign)
 
-    def extract_component(self, *indices) -> BoundaryPeriodic:
-        """extracts the boundary pair of the given extract_component.
-
-        Args:
-            *indices:
-                One or two indices for vector or tensor fields, respectively
-        """
-        return self
-
     def check_value_rank(self, rank: int) -> None:
         """check whether the values at the boundaries have the correct rank
 
@@ -391,7 +363,7 @@ class BoundaryPeriodic(BoundaryPair):
 
 
 def get_boundary_axis(
-    grid: GridBase, axis: int, data, rank: int = 0, normal: bool = False
+    grid: GridBase, axis: int, data, rank: int = 0
 ) -> BoundaryAxisBase:
     """return object representing the boundary condition for a single axis
 
@@ -404,9 +376,6 @@ def get_boundary_axis(
             Data describing the boundary conditions for this axis
         rank (int):
             The tensorial rank of the field for this boundary condition
-        normal (bool):
-            Flag indicating whether the condition is only applied in the normal
-            direction.
 
     Returns:
         :class:`~pde.grids.boundaries.axis.BoundaryAxisBase`:
@@ -435,7 +404,7 @@ def get_boundary_axis(
         bcs = BoundaryPeriodic(grid, axis)
     else:
         # initialize independent boundary conditions for the two sides
-        bcs = BoundaryPair.from_data(grid, axis, data, rank=rank, normal=normal)
+        bcs = BoundaryPair.from_data(grid, axis, data, rank=rank)
 
     # check consistency
     if bcs.periodic != grid.periodic[axis]:
