@@ -10,6 +10,8 @@ vanishes.
  
 """
 
+from __future__ import annotations
+
 import warnings
 from abc import ABCMeta
 from typing import TYPE_CHECKING, Any, Dict, Generator, Tuple, TypeVar, Union
@@ -17,7 +19,6 @@ from typing import TYPE_CHECKING, Any, Dict, Generator, Tuple, TypeVar, Union
 import numpy as np
 
 from ..tools.cache import cached_property
-from ..tools.docstrings import fill_in_docstring
 from ..tools.plotting import plot_on_axes
 from .base import DimensionError, GridBase, _check_shape, discretize_interval
 from .cartesian import CartesianGrid
@@ -117,13 +118,8 @@ class SphericalSymGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equ
         """state: the state of the grid"""
         return {"radius": self.radius, "shape": self.shape}
 
-    @property
-    def has_hole(self) -> bool:
-        """returns whether the inner radius is larger than zero"""
-        return self.axes_bounds[0][0] > 0
-
     @classmethod
-    def from_state(cls, state: Dict[str, Any]) -> "SphericalSymGridBase":  # type: ignore
+    def from_state(cls, state: Dict[str, Any]) -> SphericalSymGridBase:  # type: ignore
         """create a field from a stored `state`.
 
         Args:
@@ -136,6 +132,33 @@ class SphericalSymGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equ
             raise ValueError(f"State items {state_copy.keys()} were not used")
         return obj
 
+    @classmethod
+    def from_bounds(  # type: ignore
+        cls,
+        bounds: Tuple[Tuple[float, float]],
+        shape: Tuple[int],
+        periodic: Tuple[bool],
+    ) -> SphericalSymGridBase:
+        """
+        Args:
+            bounds (tuple): Give the coordinate range for the radial axis.
+            shape (tuple): The number of support points for the radial axis
+            periodic (bool or list): Not used
+
+        Returns:
+            SphericalGridBase representing the region chosen by bounds
+        """
+        if len(bounds) != 1:
+            raise ValueError(
+                f"`bounds` must be given as ((r_min, r_max),). Got {bounds} instead"
+            )
+        return cls(bounds[0], shape)
+
+    @property
+    def has_hole(self) -> bool:
+        """returns whether the inner radius is larger than zero"""
+        return self.axes_bounds[0][0] > 0
+
     @property
     def radius(self) -> Union[float, Tuple[float, float]]:
         """float: radius of the sphere"""
@@ -391,68 +414,6 @@ class SphericalSymGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equ
         else:
             return rs
 
-    @fill_in_docstring
-    def get_boundary_conditions(
-        self, bc="auto_periodic_neumann", rank: int = 0, normal: bool = False
-    ) -> "Boundaries":
-        """constructs boundary conditions from a flexible data format.
-
-        If the inner boundary condition for a grid without a hole is not specified, this
-        condition is automatically set to a vanishing derivative at :math:`r=0`.
-
-        Args:
-            bc (str or list or tuple or dict):
-                The boundary conditions applied to the field.
-                {ARG_BOUNDARIES}
-            rank (int):
-                The tensorial rank of the value associated with the boundary
-                conditions.
-            normal (bool):
-                Flag indicating whether the condition is only applied in the normal
-                direction.
-
-        Raises:
-            ValueError: If the data given in `bc` cannot be read
-            PeriodicityError: If the boundaries are not compatible with the
-                periodic axes of the grid.
-        """
-        from .boundaries import Boundaries  # @Reimport
-        from .boundaries.axis import BoundaryPair
-        from .boundaries.local import BCBase, NeumannBC
-
-        if self.has_hole:
-            # grid has holes => specify two boundary conditions
-            return Boundaries.from_data(self, bc, rank=rank, normal=normal)
-
-        if isinstance(bc, Boundaries):
-            # a full boundary instance is given
-            return bc
-
-        if bc == "auto_periodic_neumann" or bc == "auto_periodic_neumann":
-            # a simple value is given => use it for the outer boundary
-            return Boundaries.from_data(self, "derivative", rank=rank, normal=normal)
-        elif bc == "auto_periodic_dirichlet":
-            # a simple value is given => use it for the outer boundary
-            return Boundaries.from_data(self, "value", rank=rank, normal=normal)
-        else:
-            # a more complex value is given for the boundary
-            try:
-                # try interpreting it as a value for the outer boundary
-                b_outer = BCBase.from_data(
-                    self, 0, upper=True, data=bc, rank=rank, normal=normal
-                )
-            except ValueError:
-                # if this fails, try interpreting the value as the full BC
-                bcs = Boundaries.from_data(self, bc, rank=rank, normal=normal)
-            else:
-                self._logger.warning(
-                    "The inner boundary condition was not specified. Assuming a "
-                    "vanishing derivative at r=0."
-                )
-                b_inner = NeumannBC(self, 0, upper=False, rank=rank)
-                bcs = Boundaries([BoundaryPair(b_inner, b_outer)])
-            return bcs
-
     def get_cartesian_grid(self, mode: str = "valid", num: int = None) -> CartesianGrid:
         """return a Cartesian grid for this spherical one
 
@@ -472,7 +433,7 @@ class SphericalSymGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equ
         # pick how the grid is determined
         if mode == "valid":
             if self.has_hole:
-                self._logger.warn("The sphere has holes, so not all points are valid")
+                self._logger.warning("Sphere has holes, so not all points are valid")
             bounds = self.radius / np.sqrt(self.dim)
         elif mode == "inscribed":
             bounds = self.radius / np.sqrt(self.dim)
