@@ -2,9 +2,16 @@
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
 
+import numba as nb
 import numpy as np
 
-from pde.tools.numba import flat_idx, jit_allocate_out, numba_environment
+from pde.tools.numba import (
+    JIT_COUNT,
+    Counter,
+    flat_idx,
+    jit_allocate_out,
+    numba_environment,
+)
 
 
 def test_environment():
@@ -26,10 +33,15 @@ def test_jit_allocate_out_1arg():
         out[:] = arr
         return out
 
+    jit_count = int(JIT_COUNT)
     a = np.linspace(0, 1, 3)
     g = jit_allocate_out(out_shape=a.shape)(f)
     np.testing.assert_equal(g(a), a)
     np.testing.assert_equal(jit_allocate_out(f)(a), a)
+    if nb.config.DISABLE_JIT:
+        assert int(JIT_COUNT) == jit_count
+    else:
+        assert int(JIT_COUNT) == jit_count + 2
 
 
 def test_jit_allocate_out_2arg():
@@ -39,9 +51,32 @@ def test_jit_allocate_out_2arg():
         out[:] = a + b
         return out
 
+    jit_count = int(JIT_COUNT)
     a = np.linspace(0, 1, 3)
     b = np.linspace(1, 2, 3)
     c = np.linspace(1, 3, 3)
     g = jit_allocate_out(out_shape=a.shape, num_args=2)(f)
     np.testing.assert_equal(g(a, b), c)
     np.testing.assert_equal(jit_allocate_out(num_args=2)(f)(a, b), c)
+    if nb.config.DISABLE_JIT:
+        assert int(JIT_COUNT) == jit_count
+    else:
+        assert int(JIT_COUNT) == jit_count + 2
+
+
+def test_counter():
+    """test Counter implementation"""
+    c1 = Counter()
+    assert int(c1) is 0
+    assert c1 == 0
+    assert str(c1) == "0"
+
+    c1.increment()
+    assert int(c1) is 1
+
+    c1 += 2
+    assert int(c1) is 3
+
+    c2 = Counter(3)
+    assert c1 is not c2
+    assert c1 == c2
