@@ -15,7 +15,6 @@ Functions for creating movies of simulation results
 """
 
 import pathlib
-import warnings
 from typing import Any, Dict
 
 from ..storage.base import StorageBase
@@ -214,7 +213,6 @@ def movie(
     filename: str,
     *,
     progress: bool = True,
-    dpi: float = 150,
     show_time: bool = True,
     plot_args: Dict[str, Any] = None,
     movie_args: Dict[str, Any] = None,
@@ -229,8 +227,6 @@ def movie(
             the format used.
         progress (bool):
             Flag determining whether the progress of making the movie is shown.
-        dpi (float):
-            Resolution of the movie
         show_time (bool):
             Whether to show the simulation time in the movie
         plot_args (dict):
@@ -245,14 +241,6 @@ def movie(
     if movie_args is None:
         movie_args = {}
 
-    # use the dpi from movie_args if present
-    movie_args.setdefault("dpi", dpi)
-    if dpi != 150:
-        warnings.warn(
-            "Setting DPI using `dpi` argument is deprecated. Use `movie_args` instead.",
-            DeprecationWarning,
-        )
-
     # create the iterator over the data
     field_iter = display_progress(storage.items(), total=len(storage), enabled=progress)
 
